// Libraries
#include <math.h>               // pow, log
#include <istream.h>            // cout, cin, endl
#include <fstream.h>            // ofstream
#include <sstream>              // stringstream, string

using std::string;
using std::stringstream;

// COnstants needed for function ran
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


// Needed for initializing random seed
long *seed;
long SEED;

// Function Declarations
double UniRand(void);						   // Draws a U[0,1]	NB: To be replaced
double DiscreteUniform(long);				   // Draws a Random Number from {0,...,m-1}
double Bernoulli(double);	                   //  Draws a Bernoulli(p)
double ran(long *idum);						   // Draws a U{0,1}
string itos(int);	                           // convert int to string

// Include Cell Header
#include "Cell.h"

// Main

int main(int argc, char *argv[])
{

        int i,j,t,h;
        
       // Initialize seed
    	seed=&SEED;
    	cout << "Enter Seed (Negative)= ";
    	cin >> SEED;
    	srand(SEED);

        // Creates N objects from class Cell
        int N, NUMCONF;
        cout << "Enter No Cells= ";
        cin >> N;
        Cell *mycell = new Cell[N];
        
        // Enter NEIGHBORS
        int NEIGHBORS;
        cout << "Enter Number of Neighbors = ";
        cin >> NEIGHBORS;
        NUMCONF = int(pow(2,NEIGHBORS+1));
        
        // Enter NUMRULE
        int NUMRULE;
        cout << "Enter Number of Rule = ";
        cin >> NUMRULE;
        
        // Enter MAXTIME
        int MAXTIME;
        cout << "Enter Time Periods  = ";
        cin >> MAXTIME;

    	// Build a string for output FILENAME "CA_" + NUMRULE
        char FILENAME[80];
        strcpy(FILENAME,"CA_");
    	string ss1=itos(NUMRULE);
    	const char* pp1 = ss1.c_str();
    	strcat(FILENAME,pp1);    
    	strcat(FILENAME,".txt");

        // Create interaction structure
        
        // Define RADIUS
        int RADIUS=NEIGHBORS/2;
        
        // Define enlarged labels
        int LAB[3*N];
        for (i=0;i<N;i++)
        {
            LAB[i]=i;
            LAB[N+i]=i;
            LAB[2*N+i]=i;       
        }

        // Open output file
        ofstream outfile(FILENAME);
         
        // Initialize Cell Properties
        for (i=0;i<N;i++)
        {

            // Set neighborhood variables
            mycell[i].set_myneighborhood_vars(NEIGHBORS+1);

            // Cell coordinates
             mycell[i].coord=i;

            // Initialize Cell status

            // Random initial condition
            //mycell[i].status=Bernoulli(0.5);

            // All zero but N/2
            if (i==N/2)
                {mycell[i].status=1;}
            else
                {mycell[i].status=0;}

            // Print initial configuration
            outfile <<  mycell[i].status << "\t";

            // Defining Neighbors 
            for (h=-RADIUS;h<=RADIUS;h++)
                {
                    mycell[i].myneigh[h+RADIUS]=LAB[N+mycell[i].coord+h];
                    mycell[i].mylocalconf[h+RADIUS]=0;    
                }
                       
        }
        outfile << endl;
        
        // Defining Initial local configurations            
        for (i=0;i<N;i++)
        {           
                for (j=0;j<NEIGHBORS+1;j++)
                {               
                    int neighbor;
                    neighbor=mycell[i].myneigh[j];
                    mycell[i].mylocalconf[j]=mycell[neighbor].status;
                }
        }


        // Setting cell rules (possibly heterogeneous)
        for (i=0;i<N;i++)
        {
            mycell[i].rule_number=NUMRULE;         // Set homogeneous rules
            mycell[i].set_rule(NUMRULE,NUMCONF);                                    
        }
        
        // Main Iteration
        
        t=1;
        
        while(t<=MAXTIME)
        {        
            
            // Updating Status
            for (i=0;i<N;i++)
            {
                mycell[i].update_status();
                outfile <<  mycell[i].status << "\t"; 
            }
            outfile << endl;
            
            // Updating Initial local configurations            
            for (i=0;i<N;i++)
            {           
                for (j=0;j<NEIGHBORS+1;j++)
                {               
                    int neighbor;
                    neighbor=mycell[i].myneigh[j];
                    mycell[i].mylocalconf[j]=mycell[neighbor].status;
                }
            }

            // Go to next generation
            t++;        
        }


        outfile.close();
        system ("pause");
        return 0;

}



// Draws a U[0,1]	NB: To be replaced
double UniRand(void)
{
	// Draws a uniform[0,1]
	//return double(rand()%RAND_MAX)/RAND_MAX;
	return ran(seed); // Use ran(*idum) in "ran.cpp"
}


// Draws a Random Number from {0,...,m-1}
double DiscreteUniform(long k)
{
	// Draws a Random Number from {0,...,k-1}
    return double(rand()%k);    
}

// Draws a realization {0,1} from a Bernoulli(p)
double Bernoulli(double bernoulli_prob)
{
	double value=UniRand();
	if (value<bernoulli_prob)
	{
		return 1.0;
	}
	else
	{
		return 0.0;
	}
}

double ran(long *idum)
/*Minimal" random number generator of Park and Miller with Bays-Durham shue and added
safeguards. Returns a uniform random deviate between 0.0 and 1.0 (exclusive of the endpoint
values). Call with *idum a negative integer to initialize; thereafter, do not alter *idum between
successive deviates in a sequence. RNMX should approximate the largest doubleing value that is
less than 1.*/
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	double temp;
	if (*idum <= 0 || !iy) {    //Initialize.
	 if (-(*idum) < 1) *idum=1; //Be sure to prevent *idum = 0.
	 else *idum = -(*idum);
	 for (j=NTAB+7;j>=0;j--) {  //Load the shue table (after 8 warm-ups).
	   k=(*idum)/IQ;
	   *idum=IA*(*idum-k*IQ)-IR*k;
	   if (*idum < 0) *idum += IM;
	   if (j < NTAB) iv[j] = *idum;
	 }
	 iy=iv[0];
	}
	k=(*idum)/IQ;               //Start here when not initializing.
	*idum=IA*(*idum-k*IQ)-IR*k; //Compute *idum=(IA*idum) % IM without over-flows by Schrage's method.
	if (*idum < 0) *idum += IM;
	j=iy/NDIV;                  //Will be in the range 0..NTAB-1.
	iy=iv[j];                   //Output previously stored value and rell the
								//shuffle table.
	iv[j] = *idum;
	if ((temp=AM*iy) > RNMX) return RNMX; //Because users don't expect endpoint values.
	else return temp;
}


// Converts a int into a string
string itos(int i)	// convert int to string
	{
		stringstream s;
		s << i;
		return s.str();
	}
